@@ -92,23 +92,28 @@
 //!
 //! Here's a working version using [`TCell`] instead:
 //!
-//! ```
-//!# use qcell::{TCell, TCellOwner};
-//!# use std::rc::Rc;
-//! struct Marker;
-//! type ACell<T> = TCell<Marker, T>;
-//! type ACellOwner = TCellOwner<Marker>;
-//! let mut owner = ACellOwner::new();
-//!
-//! let item = Rc::new(ACell::new(Vec::<u8>::new()));
-//! let iref = owner.rw(&item);
-//! iref.push(1);
-//! test(&mut owner, &item);
-//!
-//! fn test(owner: &mut ACellOwner, item: &Rc<ACell<Vec<u8>>>) {
-//!     owner.rw(&item).push(2);
-//! }
-//! ```
+#![cfg_attr(
+    feature = "std",
+    doc = "
+ ```
+# use qcell::{TCell, TCellOwner};
+# use std::rc::Rc;
+ struct Marker;
+ type ACell<T> = TCell<Marker, T>;
+ type ACellOwner = TCellOwner<Marker>;
+ let mut owner = ACellOwner::new();
+
+ let item = Rc::new(ACell::new(Vec::<u8>::new()));
+ let iref = owner.rw(&item);
+ iref.push(1);
+ test(&mut owner, &item);
+
+ fn test(owner: &mut ACellOwner, item: &Rc<ACell<Vec<u8>>>) {
+     owner.rw(&item).push(2);
+ }
+ ```
+"
+)]
 //!
 //! And the same thing again using [`LCell`]:
 //!
@@ -167,6 +172,7 @@
 //!
 //! - Pro: Simple
 //! - Pro: Allows very flexible borrowing patterns
+//! - Pro: `no_std` support
 //! - Con: No compile-time borrowing checks
 //! - Con: Can panic due to distant code changes
 //! - Con: Runtime borrow checks and some cell space overhead
@@ -177,6 +183,7 @@
 //! - Pro: Compile-time borrowing checks
 //! - Pro: Dynamic owner creation, not limited in any way
 //! - Pro: No lifetime annotations or type parameters required
+//! - Pro: `no_std` support
 //! - Con: Can only borrow up to 3 objects at a time
 //! - Con: Runtime owner checks and some cell space overhead
 //!
@@ -190,6 +197,7 @@
 //! (TLCell), meaning only one owner is allowed per thread or process
 //! per marker type.  Code intended to be nested on the call stack
 //! must be parameterised with an external marker type.
+//! - Con: Currently requires `std`
 //!
 //! [`LCell`] pros and cons:
 //!
@@ -197,13 +205,14 @@
 //! - Pro: No overhead at runtime for borrowing or ownership checks
 //! - Pro: No cell space overhead
 //! - Pro: No need for singletons, meaning that one use does not limit other nested uses
+//! - Pro: `no_std` support
 //! - Con: Can only borrow up to 3 objects at a time
 //! - Con: Requires lifetime annotations on calls and structures
 //!
 //! Cell | Owner ID | Cell overhead | Borrow check | Owner check | Owner creation check
 //! ---|---|---|---|---|---
 //! `RefCell` | n/a | `usize` | Runtime | n/a | n/a
-//! `QCell` | integer | `u32` | Compile-time | Runtime | Runtime
+//! `QCell` | integer | `usize` | Compile-time | Runtime | Runtime
 //! `TCell` or `TLCell` | marker type | none | Compile-time | Compile-time | Runtime
 //! `LCell` | lifetime | none | Compile-time | Compile-time | Compile-time
 //!
@@ -230,9 +239,9 @@
 //! Cell | Cell immutable borrow | Cell mutable borrow
 //! ---|---|---
 //! `RefCell` | `cell.borrow()` | `cell.borrow_mut()`
-//! `QCell` | `owner.ro(&cell)` | `owner.rw(&cell)`
-//! `TCell` or `TLCell` | `owner.ro(&cell)` | `owner.rw(&cell)`
-//! `LCell` | `owner.ro(&cell)` | `owner.rw(&cell)`
+//! `QCell` | `cell.ro(&owner)` or<br/>`owner.ro(&cell)` | `cell.rw(&mut owner)` or<br/>`owner.rw(&cell)`
+//! `TCell` or `TLCell` | `cell.ro(&owner)` or<br/>`owner.ro(&cell)` | `cell.rw(&mut owner)` or<br/>`owner.rw(&cell)`
+//! `LCell` | `cell.ro(&owner)` or<br/>`owner.ro(&cell)` | `cell.rw(&mut owner)` or<br/>`owner.rw(&cell)`
 //!
 //! # Multi-threaded use: Send and Sync
 //!
@@ -302,6 +311,19 @@
 //! `QCellOwner`, then you get access to the `T` instances essentially
 //! for free.
 //!
+//! # `no_std` support
+//!
+//! There are three levels at which **qcell** crate can be built:
+//!
+//! - Full `std` support, which is the default
+//!
+//! - `no_std` with `alloc`, when built with `--no-default-features`
+//! and `--features alloc`
+//!
+//! - `no_std` without `alloc`, when built with `--no-default-features`
+//!
+//! Both [`QCell`] and [`LCell`] support all three levels.
+//!
 //! # Origin of names
 //!
 //! "Q" originally referred to quantum entanglement, the idea being
@@ -330,16 +352,27 @@
 //! [**Migi**]: https://github.com/Migi
 //! [**pythonesque**]: https://github.com/pythonesque
 
+#![cfg_attr(not(any(feature = "std", test)), no_std)]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![deny(rust_2018_idioms)]
 
+#[cfg(feature = "alloc")]
+extern crate alloc;
+
 mod lcell;
 mod qcell;
+#[cfg(feature = "std")]
 mod tcell;
+#[cfg(feature = "std")]
 mod tlcell;
 
 pub mod doctest_lcell;
+#[cfg(feature = "alloc")]
 pub mod doctest_qcell;
+pub mod doctest_qcell_noalloc;
+#[cfg(feature = "std")]
 pub mod doctest_tcell;
+#[cfg(feature = "std")]
 pub mod doctest_tlcell;
 
 // Used in LCell, TCell and TLCell.  See the Rustonomicon chapters
@@ -366,12 +399,15 @@ struct Invariant<T>(fn(T) -> T);
 pub use crate::lcell::LCell;
 pub use crate::lcell::LCellOwner;
 pub use crate::qcell::QCell;
-pub use crate::qcell::QCellOwner;
 pub use crate::qcell::QCellOwnerID;
-pub use crate::tcell::TCell;
-pub use crate::tcell::TCellOwner;
-pub use crate::tlcell::TLCell;
-pub use crate::tlcell::TLCellOwner;
+pub use crate::qcell::QCellOwnerPinned;
+pub use crate::qcell::QCellOwnerSeq;
+
+#[cfg(feature = "alloc")]
+pub use crate::qcell::QCellOwner;
+
+#[cfg(feature = "std")]
+pub use crate::{tcell::TCell, tcell::TCellOwner, tlcell::TLCell, tlcell::TLCellOwner};
 
 // The compile-tests double-check that the compile_fail tests in the
 // doctests actually fail for the reason intended, not for some other
@@ -390,11 +426,20 @@ pub use crate::tlcell::TLCellOwner;
 // all is okay, check in the changes.
 #[cfg(test)]
 pub mod compiletest {
-    #[rustversion::all(stable, since(1.55), before(1.56))]
+    #[rustversion::all(stable, since(1.58), before(1.59))]
     #[test]
     fn ui() {
         let t = trybuild::TestCases::new();
-        t.compile_fail("src/compiletest/*.rs");
+        if cfg!(feature = "std") {
+            t.compile_fail("src/compiletest/*.rs");
+        } else if cfg!(feature = "alloc") {
+            t.compile_fail("src/compiletest/lcell-*.rs");
+            t.compile_fail("src/compiletest/qcell_noalloc-*.rs");
+            t.compile_fail("src/compiletest/qcell-*.rs");
+        } else {
+            t.compile_fail("src/compiletest/lcell-*.rs");
+            t.compile_fail("src/compiletest/qcell_noalloc-*.rs");
+        }
     }
 }
 
